@@ -1,4 +1,3 @@
-require 'rchardet'
 require 'tempfile'
 require 'zlib'
 
@@ -71,10 +70,12 @@ module Git
     # options:
     #   :bare
     #   :working_directory
+    #   :initial_branch
     #
     def init(opts={})
       arr_opts = []
       arr_opts << '--bare' if opts[:bare]
+      arr_opts << "--initial-branch=#{opts[:initial_branch]}" if opts[:initial_branch]
 
       command('init', arr_opts)
     end
@@ -94,9 +95,9 @@ module Git
     #
     # @return [Hash] the options to pass to {Git::Base.new}
     #
-    def clone(repository, name, opts = {})
+    def clone(repository_url, directory, opts = {})
       @path = opts[:path] || '.'
-      clone_dir = opts[:path] ? File.join(@path, name) : name
+      clone_dir = opts[:path] ? File.join(@path, directory) : directory
 
       arr_opts = []
       arr_opts << '--bare' if opts[:bare]
@@ -105,11 +106,11 @@ module Git
       arr_opts << '--config' << opts[:config] if opts[:config]
       arr_opts << '--origin' << opts[:remote] || opts[:origin] if opts[:remote] || opts[:origin]
       arr_opts << '--recursive' if opts[:recursive]
-      arr_opts << "--mirror" if opts[:mirror]
+      arr_opts << '--mirror' if opts[:mirror]
 
       arr_opts << '--'
 
-      arr_opts << repository
+      arr_opts << repository_url
       arr_opts << clone_dir
 
       command('clone', arr_opts)
@@ -418,7 +419,7 @@ module Git
 
       hsh = {}
       command_lines('grep', grep_opts).each do |line|
-        if m = /(.*)\:(\d+)\:(.*)/.match(line)
+        if m = /(.*?)\:(\d+)\:(.*)/.match(line)
           hsh[m[1]] ||= []
           hsh[m[1]] << [m[2].to_i, m[3]]
         end
@@ -485,7 +486,9 @@ module Git
       command_lines('ls-files', '--stage', location).each do |line|
         (info, file) = line.split("\t")
         (mode, sha, stage) = info.split
-        file = eval(file) if file =~ /^\".*\"$/ # This takes care of quoted strings returned from git
+        if file.start_with?('"') && file.end_with?('"')
+          file = Git::EscapedPath.new(file[1..-2]).unescape
+        end
         hsh[file] = {:path => file, :mode_index => mode, :sha_index => sha, :stage => stage}
       end
       hsh
@@ -646,7 +649,8 @@ module Git
     #  :date
     #  :no_verify
     #  :allow_empty_message
-    #  :gpg_sign
+    #  :gpg_sign (accepts true or a gpg key ID as a String)
+    #  :no_gpg_sign (conflicts with :gpg_sign)
     #
     # @param [String] message the commit message to be used
     # @param [Hash] opts the commit options to be used
@@ -660,13 +664,18 @@ module Git
       arr_opts << "--date=#{opts[:date]}" if opts[:date].is_a? String
       arr_opts << '--no-verify' if opts[:no_verify]
       arr_opts << '--allow-empty-message' if opts[:allow_empty_message]
-      if opts[:gpg_sign]
+
+      if opts[:gpg_sign] && opts[:no_gpg_sign]
+        raise ArgumentError, 'cannot specify :gpg_sign and :no_gpg_sign'
+      elsif opts[:gpg_sign]
         arr_opts <<
           if opts[:gpg_sign] == true
             '--gpg-sign'
           else
             "--gpg-sign=#{opts[:gpg_sign]}"
           end
+      elsif opts[:no_gpg_sign]
+        arr_opts << '--no-gpg-sign'
       end
 
       command('commit', arr_opts)
@@ -682,6 +691,7 @@ module Git
     def clean(opts = {})
       arr_opts = []
       arr_opts << '--force' if opts[:force]
+      arr_opts << '-ff' if opts[:ff]
       arr_opts << '-d' if opts[:d]
       arr_opts << '-x' if opts[:x]
 
@@ -754,11 +764,21 @@ module Git
       command('branch', '-D', branch)
     end
 
+    # Runs checkout command to checkout or create branch
+    #
+    # accepts options:
+    #  :new_branch
+    #  :force
+    #  :start_point
+    #
+    # @param [String] branch
+    # @param [Hash] opts
     def checkout(branch, opts = {})
       arr_opts = []
       arr_opts << '-b' if opts[:new_branch] || opts[:b]
       arr_opts << '--force' if opts[:force] || opts[:f]
       arr_opts << branch
+      arr_opts << opts[:start_point] if opts[:start_point] && arr_opts.include?('-b')
 
       command('checkout', arr_opts)
     end
@@ -772,6 +792,7 @@ module Git
 
     def merge(branch, message = nil, opts = {})
       arr_opts = []
+      arr_opts << '--no-commit' if opts[:no_commit]
       arr_opts << '--no-ff' if opts[:no_ff]
       arr_opts << '-m' << message if message
       arr_opts += [branch]
@@ -872,13 +893,18 @@ module Git
       command('tag', arr_opts)
     end
 
-
     def fetch(remote, opts)
-      arr_opts = [remote]
-      arr_opts << opts[:ref] if opts[:ref]
+      arr_opts = []
+      arr_opts << '--all' if opts[:all]
       arr_opts << '--tags' if opts[:t] || opts[:tags]
       arr_opts << '--prune' if opts[:p] || opts[:prune]
+      arr_opts << '--prune-tags' if opts[:P] || opts[:'prune-tags']
+      arr_opts << '--force' if opts[:f] || opts[:force]
       arr_opts << '--unshallow' if opts[:unshallow]
+      arr_opts << '--depth' << opts[:depth] if opts[:depth]
+      arr_opts << '--' if remote || opts[:ref]
+      arr_opts << remote if remote
+      arr_opts << opts[:ref] if opts[:ref]
 
       command('fetch', arr_opts)
     end
@@ -1001,8 +1027,9 @@ module Git
     # returns the current version of git, as an Array of Fixnums.
     def current_command_version
       output = command('version')
-      version = output[/\d+\.\d+(\.\d+)+/]
-      version.split('.').collect {|i| i.to_i}
+      version = output[/\d+(\.\d+)+/]
+      version_parts = version.split('.').collect { |i| i.to_i }
+      version_parts.fill(0, version_parts.length...3)
     end
 
     def required_command_version
@@ -1013,6 +1040,14 @@ module Git
       (self.current_command_version <=>  self.required_command_version) >= 0
     end
 
+    def self.warn_if_old_command(lib)
+      return true if @version_checked
+      @version_checked = true
+      unless lib.meets_required_version?
+        $stderr.puts "[WARNING] The git gem requires git #{lib.required_command_version.join('.')} or later, but only found #{lib.current_command_version.join('.')}. You should probably upgrade."
+      end
+      true
+    end
 
     private
 
@@ -1069,6 +1104,8 @@ module Git
     end
 
     def command(cmd, *opts, &block)
+      Git::Lib.warn_if_old_command(self)
+
       command_opts = { chomp: true, redirect: '' }
       if opts.last.is_a?(Hash)
         command_opts.merge!(opts.pop)
@@ -1080,7 +1117,8 @@ module Git
       global_opts = []
       global_opts << "--git-dir=#{@git_dir}" if !@git_dir.nil?
       global_opts << "--work-tree=#{@git_work_dir}" if !@git_work_dir.nil?
-      global_opts << ["-c", "color.ui=false"]
+      global_opts << %w[-c core.quotePath=true]
+      global_opts << %w[-c color.ui=false]
 
       opts = [opts].flatten.map {|s| escape(s) }.join(' ')
 
@@ -1171,35 +1209,10 @@ module Git
       arr_opts
     end
 
-    def default_encoding
-      __ENCODING__.name
-    end
-
-    def best_guess_encoding
-      # Encoding::ASCII_8BIT.name
-      Encoding::UTF_8.name
-    end
-
-    def detected_encoding(str)
-      CharDet.detect(str)['encoding'] || best_guess_encoding
-    end
-
-    def encoding_options
-      { invalid: :replace, undef: :replace }
-    end
-
-    def normalize_encoding(str)
-      return str if str.valid_encoding? && str.encoding.name == default_encoding
-
-      return str.encode(default_encoding, str.encoding, **encoding_options) if str.valid_encoding?
-
-      str.encode(default_encoding, detected_encoding(str), **encoding_options)
-    end
-
     def run_command(git_cmd, &block)
       return IO.popen(git_cmd, &block) if block_given?
 
-      `#{git_cmd}`.lines.map { |l| normalize_encoding(l) }.join
+      `#{git_cmd}`.lines.map { |l| Git::EncodingUtils.normalize_encoding(l) }.join
     end
 
     def escape(s)
@@ -1211,8 +1224,9 @@ module Git
     end
 
     def escape_for_windows(s)
-      # Windows does not need single quote escaping inside double quotes
-      %Q{"#{s}"}
+      # Escape existing double quotes in s and then wrap the result with double quotes
+      escaped_string = s.to_s.gsub('"','\\"')
+      %Q{"#{escaped_string}"}
     end
 
     def windows_platform?
@@ -1220,6 +1234,5 @@ module Git
       win_platform_regex = /mingw|mswin/
       RUBY_PLATFORM =~ win_platform_regex || RUBY_DESCRIPTION =~ win_platform_regex
     end
-
   end
 end
