@@ -76,11 +76,31 @@ class TestDiff < Test::Unit::TestCase
     assert_equal(1, s[:files]["scott/newfile"][:deletions])
   end
   
-  def test_diff_hashkey
+  def test_diff_hashkey_default
     assert_equal('5d46068', @diff["scott/newfile"].src)
     assert_nil(@diff["scott/newfile"].blob(:dst))
     assert(@diff["scott/newfile"].blob(:src).is_a?(Git::Object::Blob))
   end
+
+  def test_diff_hashkey_min
+    set_file_paths
+    git = Git.open(@wdir)
+    git.config('core.abbrev', 4)
+    diff = git.diff('gitsearch1', 'v2.5')
+    assert_equal('5d46', diff["scott/newfile"].src)
+    assert_nil(diff["scott/newfile"].blob(:dst))
+    assert(diff["scott/newfile"].blob(:src).is_a?(Git::Object::Blob))
+  end
+
+  def test_diff_hashkey_max
+    set_file_paths
+    git = Git.open(@wdir)
+    git.config('core.abbrev', 40)
+    diff = git.diff('gitsearch1', 'v2.5')
+    assert_equal('5d4606820736043f9eed2a6336661d6892c820a5', diff["scott/newfile"].src)
+    assert_nil(diff["scott/newfile"].blob(:dst))
+    assert(diff["scott/newfile"].blob(:src).is_a?(Git::Object::Blob))
+  end
   
   def test_patch
     p = @git.diff('v2.8^', 'v2.8').patch
