@@ -38,14 +38,17 @@ class TestInit < Test::Unit::TestCase
       assert(File.directory?(File.join(path, '.git')))
       assert(File.exist?(File.join(path, '.git', 'config')))
       assert_equal('false', repo.config('core.bare'))
+
+      branch = `git config --get init.defaultBranch`.strip
+      branch = 'master' if branch.empty?
+      assert_equal("ref: refs/heads/#{branch}\n", File.read("#{path}/.git/HEAD"))
     end
   end
 
   def test_git_init_bare
     in_temp_dir do |path|
       repo = Git.init(path, :bare => true)
-      assert(File.directory?(File.join(path, '.git')))
-      assert(File.exist?(File.join(path, '.git', 'config')))
+      assert(File.exist?(File.join(path, 'config')))
       assert_equal('true', repo.config('core.bare'))
     end
   end
@@ -61,6 +64,16 @@ class TestInit < Test::Unit::TestCase
     end
   end
 
+  def test_git_init_initial_branch
+    in_temp_dir do |path|
+      repo = Git.init(path, initial_branch: 'main')
+      assert(File.directory?(File.join(path, '.git')))
+      assert(File.exist?(File.join(path, '.git', 'config')))
+      assert_equal('false', repo.config('core.bare'))
+      assert_equal("ref: refs/heads/main\n", File.read("#{path}/.git/HEAD"))
+    end
+  end
+
   def test_git_clone
     in_temp_dir do |path|
       g = Git.clone(@wbare, 'bare-co')
