@@ -1,6 +1,7 @@
 #!/usr/bin/env ruby
 
 require File.dirname(__FILE__) + '/../test_helper'
+require "fileutils"
 
 # tests all the low level git communication
 #
@@ -19,7 +20,7 @@ class TestLib < Test::Unit::TestCase
       git = Git.clone("file://#{@wdir}", "shallow", path: dir, depth: 1).lib
       assert_equal(1,  git.log_commits.length)
       git.fetch("file://#{@wdir}", unshallow: true)
-      assert_equal(71,  git.log_commits.length)
+      assert_equal(72,  git.log_commits.length)
     end
   end
 
@@ -51,8 +52,13 @@ class TestLib < Test::Unit::TestCase
     move_file(pre_commit_path, pre_commit_path_bak)
 
     # Adds a pre-commit file that should throw an error
-    create_file(pre_commit_path, 'echo Pre-commit file. Shoud not execute; exit 1') # Error when executed
-    File.chmod(0111, pre_commit_path)
+    create_file(pre_commit_path, <<~PRE_COMMIT_SCRIPT)
+      #!/bin/sh
+      echo "pre-commit script exits with an error"
+      exit 1
+    PRE_COMMIT_SCRIPT
+
+    FileUtils.chmod("+x", pre_commit_path)
 
     create_file("#{@wdir}/test_file_2", 'content test_file_2')
     @lib.add('test_file_2')
@@ -81,6 +87,19 @@ class TestLib < Test::Unit::TestCase
     assert(@lib.checkout('master'))
   end
 
+  def test_checkout_with_start_point
+    assert(@lib.reset(nil, hard: true)) # to get around worktree status on windows
+
+    actual_cmd = nil
+    @lib.define_singleton_method(:run_command) do |git_cmd, &block|
+      actual_cmd = git_cmd
+      super(git_cmd, &block)
+    end
+
+    assert(@lib.checkout('test_checkout_b2', {new_branch: true, start_point: 'master'}))
+    assert_match(%r/checkout ['"]-b['"] ['"]test_checkout_b2['"] ['"]master['"]/, actual_cmd)
+  end
+
   # takes parameters, returns array of appropriate commit objects
   # :count
   # :since
@@ -282,10 +301,14 @@ class TestLib < Test::Unit::TestCase
     match = @lib.grep('search', :object => 'gitsearch1', :invert_match => true)
     assert_equal(6, match['gitsearch1:scott/text.txt'].size)
     assert_equal(2, match.size)
+
+    match = @lib.grep('Grep', :object => 'grep_colon_numbers')
+    assert_equal("Grep regex doesn't like this:4342: because it is bad", match['grep_colon_numbers:colon_numbers.txt'].first[1])
+    assert_equal(1, match.size)
   end
 
   def test_show
-    assert(/^commit 5e53019b3238362144c2766f02a2c00d91fcc023.+\+replace with new text - diff test$/m.match(@lib.show))
+    assert_match(/^commit 46abbf07e3c564c723c7c039a43ab3a39e5d02dd.+\+Grep regex doesn't like this:4342: because it is bad\n$/m, @lib.show)
     assert(/^commit 935badc874edd62a8629aaf103418092c73f0a56.+\+nothing!$/m.match(@lib.show('gitsearch1')))
     assert(/^hello.+nothing!$/m.match(@lib.show('gitsearch1', 'scott/text.txt')))
     assert(@lib.show('gitsearch1', 'scott/text.txt') == "hello\nthis is\na file\nthat is\nput here\nto search one\nto search two\nnothing!\n")
