@@ -130,4 +130,33 @@ class TestMerge < Test::Unit::TestCase
       end
     end
   end
+
+  def test_merge_no_commit
+    in_temp_dir do |path|
+      g = Git.clone(@wbare, 'branch_merge_test')
+      g.chdir do
+        g.branch('new_branch_1').in_branch('first commit message') do
+          new_file('new_file_1', 'foo')
+          g.add
+          true
+        end
+
+        g.branch('new_branch_2').in_branch('first commit message') do
+          new_file('new_file_2', 'bar')
+          g.add
+          true
+        end
+
+        g.checkout('new_branch_2')
+        before_merge = g.show
+        g.merge('new_branch_1', nil, no_commit: true)
+        # HEAD is the same as before.
+        assert_equal(before_merge, g.show)
+        # File has not been merged in.
+        status = g.status['new_file_1']
+        assert_equal('new_file_1', status.path)
+        assert_equal('A', status.type)
+      end
+    end
+  end
 end
